import { Component, ViewChild } from '@angular/core';

import { Message } from 'jalhyd';

import { InternationalisationService } from '../../services/internationalisation/internationalisation.service';
import { LogComponent } from '../../components/log/log.component';
import { RemousResults } from "../../results/remous-results";

@Component({
    selector: 'remous-results',
    templateUrl: './remous-results.component.html',
    styles: [`
    .result_label {
        text-align: right;
        padding-top:10px;
        padding-bottom:10px;
        padding-right:10px;   
    }
    .result_value {
        text-align: center;
        padding-left:30px;   
        padding-right:30px;   
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_2 {
        font-weight: bold;
    }
    `
    ]
})
export class RemousResultsComponent {
    private _results: RemousResults;

    /*
    * config du graphe principal
    */
    private graph1_type = 'line';
    private graph1_data = {};
    private graph1_options = {};

    /*
    * config du graphe secondaire (paramètre supplémentaire sauf Hs, Hsc, Yf, Yt, Yco)
    */
    private graph2_type = 'line';
    private graph2_data = {};
    private graph2_options = {};

    /**
     * composant journal
     */
    @ViewChild(LogComponent)
    private logComponent: LogComponent;

    constructor(private intlService: InternationalisationService) {
    }

    private get uitextLigneFluviale() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_LIGNEFLUVIALE")
    }

    private get uitextLigneTorrentielle() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_LIGNETORRENTIELLE")
    }

    private get uitextAbscisse() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_ABSCISSE")
    }

    private get uitextTirant() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_TIRANT")
    }

    private get uitextFond() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_FOND")
    }

    private get uitextBerge() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_BERGE")
    }

    private get uitextTirantNormal() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_TIRANTNORMAL")
    }

    private get uitextTirantCritique() {
        return this.intlService.localizeText("INFO_REMOUSRESULTS_TIRANTCRITIQUE")
    }

    private get extraGraph(): boolean {
        return this._results == undefined ? false : this._results.extraGraph;
    }

    private get series(): Object[] {
        return this._results == undefined ? [] : this._results.series;
    }

    private get extraParamLabel(): string {
        return this._results == undefined ? undefined : this._results.extraParamLabel;
    }

    public set results(r: RemousResults) {
        this._results = r;
        this.updateView();
    }

    public updateView() {
        this.graph1_data = {};
        this.graph1_options = {};
        this.graph2_data = {};
        this.graph2_options = {};
        this.logComponent.log = undefined;
        if (this._results != undefined) {
            this.logComponent.log = this._results.log;
            this.generateGraph();
        }
    }

    private searchObjectInArray(array: any[], key: string, value: any) {
        for (let o of array)
            if (o[key] == value)
                return o;

        return undefined;
    }

    private connectRessaut(lineFlu: LineData, lineTor: LineData) {
        let tX = lineFlu.tx.slice(0); // copie

        tX.sort((a, b) => {
            if (a > b)
                return 1;
            if (a < b)
                return -1;
            return 0;
        });

        let minXflu; // abscisse de début de la courbe fluviale
        for (let x of tX) {
            let r = this.searchObjectInArray(this._results.series, "abs", x);
            if (r["flu"] != "") {
                minXflu = x;
                break;
            }
        }

        if (minXflu != undefined && minXflu != tX[0]) {
            // la courbe fluviale ne démarre pas au début, on ajoute un point de raccord avec la ligne torrentielle

            let i = tX.indexOf(minXflu);
            let xflu = tX[i - 1];
            let yflu = lineTor.getYat(xflu);
            lineFlu.setPoint(xflu, yflu);
        }

        let maxXtor; // abscisse de fin de la courbe torrentielle
        tX.reverse();
        for (let x of tX) {
            let r = this.searchObjectInArray(this._results.series, "abs", x);
            if (r["tor"] != "") {
                maxXtor = x;
                break;
            }
        }
        tX.reverse();

        if (maxXtor != undefined && maxXtor != tX[tX.length - 1]) {
            // la courbe torrentielle ne finit pas à la fin des abscisses, on ajoute un point de raccord avec la ligne fluviale

            let i = tX.indexOf(maxXtor);
            let xflu = tX[i + 1];
            let yflu = lineFlu.getYat(xflu);
            lineTor.setPoint(xflu, yflu);
        }
    }

    private generateGraph() {
        // http://www.chartjs.org/docs/latest/charts/line.html
        // le dernier dataset de la liste datasets est dessiné en 1er

        const penteFond: number = this._results.penteFond;

        // abscisses

        let labs: number[] = [];
        let xmax = -1e8;
        for (let r of this._results.series) {
            let x = +r["abs"];
            labs.push(x);
            xmax = Math.max(x, xmax);
        }

        // init graphiques

        let gr1 = new GraphData(labs, penteFond, xmax);
        if (this._results.extraGraph)
            var gr2 = new GraphData(labs, 0, xmax);

        // ligne de fond
        gr1.drawLine(0, 0, 3, "#753F00", this.uitextFond, "#753F00");

        // ligne de berge
        gr1.drawLine(this._results.hautBerge, this._results.hautBerge, 4, "#C58F50", this.uitextBerge);

        // hauteur normale
        gr1.drawLine(this._results.hautNormale, this._results.hautNormale, 5, "#A4C537", this.uitextTirantNormal);

        // hauteur critique
        gr1.drawLine(this._results.hautCritique, this._results.hautCritique, 6, "#FF0000", this.uitextTirantCritique);

        // lignes d'eau torrentielle et fluviale

        let lineFlu = gr1.newLine(0);
        let lineTor = gr1.newLine(1);
        if (this._results.hasExtra) {
            if (this._results.extraGraph)
                var lineExtra = gr2.newLine(2);
            else
                lineExtra = gr1.newLine(2);
        }

        for (let r of this._results.series) {
            let x: number = +r["abs"];
            let yFlu: string = r["flu"];
            let yTor: string = r["tor"];

            let yExtra: string = r["extraFlu"];
            if (yExtra == "")
                yExtra = r["extraTor"];

            if (yFlu != "")
                lineFlu.mapPoint(x, +yFlu);
            if (yTor != "")
                lineTor.mapPoint(x, +yTor);
            if (yExtra != "")
                lineExtra.mapPoint(x, +yExtra);
        }

        if (this._results.hasExtra) {
            if (this._results.extraGraph)
                lineExtra.data = { label: this._results.extraParamLabel, tension: 0, spanGaps: true, borderColor: "#0093BD", pointRadius: 4 };
            else
                lineExtra.data = { label: this._results.extraParamLabel, tension: 0, fill: false, spanGaps: true, borderColor: "#C17AF0", pointRadius: 4 };
        }

        // raccordement ligne fluviale -> torrentielle pour dessiner le ressaut

        this.connectRessaut(lineFlu, lineTor);

        // ajout des données au graphique

        lineTor.data = { label: this.uitextLigneTorrentielle, tension: 0, borderColor: "#77A3CD", pointBackgroundColor: "#77A3CD", pointRadius: 4, backgroundColor: "#D1D0D4" };
        lineFlu.data = { label: this.uitextLigneFluviale, tension: 0, borderColor: "#0093BD", pointBackgroundColor: "#0093BD", pointRadius: 4, backgroundColor: "#D1D0D4" };

        this.graph1_data = gr1.data;

        this.graph1_options = {
            responsive: true,
            maintainAspectRatio: true,
            animation: {
                duration: 0
            },
            legend: {
                display: true,
                position: "right"
            },
            title: {
                display: true,
                text: this.uitextAbscisse,
                position: "bottom"
            }
        };

        if (this._results.extraGraph) {
            this.graph2_data = gr2.data;

            this.graph2_options = {
                responsive: true,
                maintainAspectRatio: true,
                animation: {
                    duration: 0
                },
                legend: {
                    display: true,
                    position: "right"
                },
                title: {
                    display: true,
                    text: this.uitextAbscisse,
                    position: "bottom"
                }
            };
        }
    }

    private getResultClass(i: number) {
        return "result_id_" + String(i & 1);
    }
}

/**
 * données pour une ligne dans le graphe
 */
class LineData {
    /**
     * abscisses
     */
    private _tx: number[];

    /**
     * orodonnées
     */
    private _ty: number[] = [];

    /**
     * graphe auquel aapartient la ligne
     */
    private _parentGraph: GraphData;

    /**
     * données fournies à ChartJS
     */
    private _data = {};

    /**
     * profondeur à laquelle est dessinée la ligne
     * les profondeurs les plus petites sont dessinées derrière les profondeurs les plus grandes
     */
    public z: number;

    constructor(gr: GraphData) {
        this._parentGraph = gr;
        this._tx = gr.tx;
        for (let i = this._tx.length - 1; i >= 0; i--)
            this._ty.push(null);
    }

    public getYat(x: number) {
        let i = this._tx.indexOf(x);
        return this._ty[i];
    }

    public setPoint(x: number, y: number) {
        let i = this._tx.indexOf(x);
        this._ty[i] = y;
    }

    public mapPoint(x: number, y: number) {
        this.setPoint(x, this._parentGraph.mapY(x, y));
    }

    public get ty() {
        return this._ty;
    }

    public get tx() {
        return this._tx;
    }

    public hasYs(): boolean {
        for (let y of this._ty)
            if (y != null)
                return true;
        return false;
    }

    public get data() {
        return this._data;
    }

    public set data(d: {}) {
        this._data = d;
        this._data["data"] = this._ty;
    }
}

class GraphData {
    /**
     * tableau des (labels) des abscisses
     */
    private _tx: number[];

    private _lines: LineData[] = [];

    /**
     * pente du fond
    */
    private _penteFond: number;

    /**
     * longueur du bief
     */
    private _longBief: number;

    constructor(tX: number[], pente: number, bief: number) {
        this._tx = tX;
        this._penteFond = pente;
        this._longBief = bief;
    }

    /**
     * crée une ligne dans le graphe
     * @param z profondeur de la lign
     */
    public newLine(z: number): LineData {
        let res = new LineData(this);
        res.z = z;
        this._lines.push(res);
        return res;
    }

    public get tx(): number[] {
        return this._tx;
    }

    /**
   * transforme une cote en tenant compte du fond
   * @param x abscisse où se trouve la cote à transformer
   * @param y cote à transformer
   */
    public mapY(x: number, y: number) {
        let d: number;
        if (this._penteFond >= 0) {
            d = this._penteFond * (this._longBief - x);
        } else {
            d = -this._penteFond * x;
        }
        return y + d;
    }

    /**
     * dessine une ligne droite
     * @param y0 y en x=0
     * @param ymax y en x=xmax
     * @param color couleur de la ligne
     * @param lbl légende de la ligne
     * @param fillColor couleur de remplissage sous la ligne
     */
    public drawLine(y0: number, ymax: number, prof: number, color: string, lbl: string, fillColor: string = undefined) {
        let l = this.newLine(prof);
        l.mapPoint(0, y0);
        l.mapPoint(this._longBief, ymax);

        // l.data = { label: lbl, data: l, fill: fillColor != undefined, tension: 0, borderColor: color, backgroundColor: fillColor, pointRadius: 0 };
        l.data = {
            label: lbl, fill: fillColor != undefined, tension: 0, spanGaps: true,
            borderColor: color, backgroundColor: fillColor, pointRadius: 0
        };
    }

    public get data() {
        let ds = [];
        this._lines.sort((a, b) => {
            if (a.z > b.z)
                return -1;
            if (a.z < b.z)
                return 1;
            return 0;
        });

        for (let l of this._lines)
            ds.push(l.data);

        return {
            labels: this._tx,
            datasets: ds
        };
    }
}
